//////////////////////////////////////////////////////////////////////////
///
/// @file ParallelsTypes.h
///
/// @brief Base Parallels types declarations
///
/// @author maximk, misha
///
/// Ideally, all user - and - kernel space components are
/// supposed to use types, declared in this module.
/// Does not need any additional headers.
///
/// Copyright (c) 2006 Parallels Inc.
/// All Rights Reserved.
/// http://www.parallels.com
///
//////////////////////////////////////////////////////////////////////////

#ifndef __PARALLELS_TYPES_H__
#define __PARALLELS_TYPES_H__

// Force compilation errors if the platform defines are in conflict states
// (e.g. _X86_ is defined for __x86_64__)
// #define FORCE_PLATFORM_CHECK

// Determining current architecture
#if defined(__x86_64__) || defined(_M_X64) || defined (_AMD64_) || defined(_WIN64) || defined(_M_AMD64) || defined(_M_IA64)
#ifndef _64BIT_
	#define _64BIT_
#endif
#ifndef _AMD64_
	#define _AMD64_
#endif
#if defined(FORCE_PLATFORM_CHECK) && defined(_X86_)
	#error "Define _X86_ is incompatible with 64-bit platform!"
#endif
#elif defined(_X86_) || defined(__i386__) || defined(_M_IX86)
#ifndef _32BIT_
	#define _32BIT_
#endif
#ifndef _X86_
	#define _X86_
#endif
#if defined(FORCE_PLATFORM_CHECK) && defined(_AMD64_)
	#error "Define _AMD64_ is incompatible with 32-bit platform!"
#endif
#elif defined(__arm__)
#ifndef _32BIT_
	#define _32BIT_
#endif
#else
	#error "Failed to determine processor architecture"
#endif

#define CACHELINE_SIZE	128

/**
 * Base types declaration, used by all Parallels software,
 * including user and kernel space components.
 */

/* Don't declare BOOL for obj-c */
#ifndef OBJC_BOOL_DEFINED
typedef	int				BOOL;
#endif

typedef char			CHAR;
typedef unsigned char	UCHAR;
typedef	unsigned char	BYTE;
typedef	short			SHORT;
typedef	unsigned short	USHORT;
typedef	unsigned short	WORD;
typedef int				INT;
typedef unsigned int	UINT;
typedef	long			LONG;
typedef unsigned long	ULONG;
typedef	unsigned long	DWORD;

#define BITS_PER_UINT	(sizeof(UINT) * 8)

/**
 * There are some differences in 64-bit variables declarations
 * on windows and unix compilers.
 */
#ifdef _WIN_
	typedef __int64			LONG64;
	typedef unsigned __int64	ULONG64;
	typedef ULONG64				UINT64;
	typedef unsigned __int64	QWORD;

#else
	#undef LONG64
	#undef ULONG64
	typedef unsigned long long	__int64;
	typedef long long		LONG64;
	typedef unsigned long long	ULONG64;
	typedef ULONG64				UINT64;
	typedef unsigned long long	QWORD;
#endif

typedef unsigned int ULONG32;

#if defined (_32BIT_)
	typedef	unsigned long	SIZE_T;
	typedef	unsigned long	ULONG_PTR;
	typedef signed long     LONG_PTR;
#elif defined (_64BIT_)
	typedef	ULONG64			SIZE_T;
	typedef	ULONG64			ULONG_PTR;
	typedef LONG64			LONG_PTR;
#endif

/*
 * Compiler-dependent suffixes for 64-bit constants.
 */
#ifdef __GNUC__
	// gcc
	#define L64(x)	(x##LL)
	#define UL64(x)	(x##ULL)
#elif _MSC_VER
	// icc and cl
	#define L64(x)	(x##i64)
	#define UL64(x)	(x##ui64)
#else
	#define L64(x)	(x)
	#define UL64(x)	(x)
#endif

/**
 * The history of these 2 macros is very strage.
 * The problem is that stupid COMPILERS like gcc, msvc, icc
 * tend to perfrom 32->64 bit conversion of unsigned values
 * using signed !!! extension.
 *
 * Found no other way then to use these macroses...
 */
#define PTR_TO_64(ptr, _64) { \
	if (sizeof(void*) == 4) { \
		_64 = ((ULONG64)ptr & 0x00000000ffffffff); \
	} else { \
		_64 = (ULONG64)ptr; \
	}}

typedef UINT HYPERSTATUS;

#define HYP_STATUS_SUCCESS		    	(0x00000000)
#define HYP_INSUFFICIENT_RESOURCES		(0x80000001)
#define HYP_INVALID_PARAMETERS			(0x80000002)
#define HYP_UNKNOWN_HPC_CALL			(0x80000003)
#define HYP_INVALID_DEVICE_STATE		(0x80000004)
#define HYP_ALREADY_MAPPED			    (0x80000005)
#define HYP_SYSTEM_OVERCOMMIT			(0x80000006)
#define HYP_MIN_WSET_REACHED			(0x80000007)
#define HYP_TOO_SMALL_BUFFER			(0x80000008)
#define HYP_NOT_CANONICAL_ADDRESS		(0x80000009)
#define HYP_INTERNAL_INCOHERENCE		(0x8000000a)
#define HYP_STATUS_UNSUCCESSFUL			(0xffffffff)


/**
 * Declaring pointers to the base classes - we're not sure if
 * this is a good idea to keep this, but many developers already use it.
 */
#define VOID		void

typedef VOID		*PVOID;
typedef VOID		*LPVOID;

typedef BOOL		*PBOOL;
typedef CHAR		*PCHAR;
typedef UCHAR		*PUCHAR;
typedef BYTE		*PBYTE;
typedef SHORT		*PSHORT;
typedef USHORT		*PUSHORT;
typedef WORD		*PWORD;
typedef INT			*PINT;
typedef UINT		*PUINT;
typedef LONG		*PLONG;
typedef	DWORD		*PDWORD;
typedef ULONG		*PULONG;
typedef ULONG32		*PULONG32;
typedef LONG64		*PLONG64;
typedef ULONG64		*PULONG64;
typedef ULONG_PTR	*PULONG_PTR;


typedef UCHAR		UINT8,  *PUINT8;
typedef USHORT		UINT16, *PUINT16;
typedef SHORT		INT16,  *PINT16;
typedef UINT		UINT32, *PUINT32;
typedef INT			INT32,  *PINT32;


/**
 * Some very common declarations - not sure where to keep them,
 * this place seems to be the best one
 */
#ifndef PAGE_SIZE
	#define PAGE_SIZE	0x1000
#endif

#define CPUMASK_ALL		((UINT)0xFFFFFFFF)
#define CPUMASK_NONE	0

#ifndef TRUE
	#define TRUE	1
#endif

#ifndef FALSE
	#define FALSE	0
#endif

#ifndef NULL
	#define NULL	0
#endif

#ifndef MAX
	#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#endif

#ifndef MIN
	#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#endif


/**
 * For platforms compatibility, we need to to have some
 * very common macros one each platform.
 */
#ifdef _MSC_VER

	// Visual Studio 2005 warnings madness workaround
	// (eliminating "deprecation" warnings)
	#ifndef _CRT_SECURE_NO_DEPRECATE
		#define _CRT_SECURE_NO_DEPRECATE
	#endif

	#ifndef UNUSED
		#define UNUSED
	#endif // UNUSED
	#define PACKED
	#define UNUSED_PARAM(x) (void)x
	#define STRUCT_NAME(x) x

#elif defined(__GNUC__)

	#ifndef UNUSED
		#define UNUSED __attribute__((unused))
	#endif // UNUSED
	#define PACKED __attribute__((packed))
	#define UNUSED_PARAM(x) (void)x
	#define STRUCT_NAME(x) PACKED x

#else

	#error Unknown compiler detected (!)

#endif

#endif // __PARALLELS_TYPES_H__
