/*-
 * Copyright (c) 2009, Alexander Nedotsukov
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*-
 * This driver was ported from Parallels Linux Guest Tools
 * Copyright (C) 2008 Parallels Software Inc.
 * All Rights Reserved.
 * http://www.parallels.com
 */

#include "ApiNet.h"
#include "OEMNetInterface.h"

struct pvmnet_softc {
	struct ifnet		*ifp;        /* interface info */
	device_t		dev;
	struct mtx		sc_mtx;

	int			irq_rid;
	struct resource		*irq_res;
	void			*irq_handle;
	int			port_rid;
	struct resource		*port_res;
	bus_space_tag_t		port_bst;
	bus_space_handle_t	port_bsh;
	int			mem_rid;
	struct resource		*mem_res;
	bus_space_tag_t		mem_bst;
	bus_space_handle_t	mem_bsh;
	u_char			enaddr[ETHER_ADDR_LEN];
};

#define PVMNET_MUTEX(sc)	(&(sc)->sc_mtx)
#define PVMNET_LOCK(sc)		mtx_lock(PVMNET_MUTEX(sc))
#define PVMNET_UNLOCK(sc)	mtx_unlock(PVMNET_MUTEX(sc))
#define PVMNET_LOCK_INIT(sc) \
	mtx_init(PVMNET_MUTEX(sc), device_get_nameunit((sc)->dev), \
		MTX_NETWORK_LOCK, MTX_DEF)
#define PVMNET_LOCK_DESTROY(sc) \
	mtx_destroy(PVMNET_MUTEX(sc))
#define PVMNET_ASSERT_LOCKED(sc) mtx_assert(PVMNET_MUTEX(sc), MA_OWNED)
#define PVMNET_ASSERT_UNLOCKED(sc) mtx_assert(PVMNET_MUTEX(sc), MA_NOTOWNED)

#define pvmnet_inw(sc, port) \
	bus_space_read_2((sc)->port_bst, (sc)->port_bsh, (port))
#define pvmnet_outw(sc, port, value) \
	bus_space_write_2((sc)->port_bst, (sc)->port_bsh, (port), (value))
#define pvmnet_inl(sc, port) \
	bus_space_read_4((sc)->port_bst, (sc)->port_bsh, (port))
#define pvmnet_outl(sc, port, value) \
	bus_space_write_4((sc)->port_bst, (sc)->port_bsh, (port), (value))

#define pvmnet_insb(sc, port, addr, count) \
	bus_space_read_multi_1((sc)->port_bst, (sc)->port_bsh, (port), (addr), (count))
#define pvmnet_outsb(sc, port, addr, count) \
	bus_space_write_multi_1((sc)->port_bst, (sc)->port_bsh, (port), (addr), (count))

#define pvmnet_readw(sc, port) \
	bus_space_read_2((sc)->mem_bst, (sc)->mem_bsh, (port))
#define pvmnet_writew(sc, port, value) \
	bus_space_write_2((sc)->mem_bst, (sc)->mem_bsh, (port), (value))
#define pvmnet_readl(sc, port) \
	bus_space_read_4((sc)->mem_bst, (sc)->mem_bsh, (port))
#define pvmnet_writel(sc, port, value) \
	bus_space_write_4((sc)->mem_bst, (sc)->mem_bsh, (port), (value))

#define pvmnet_read(sc, port, addr, count) \
	bus_space_read_region_1((sc)->mem_bst, (sc)->mem_bsh, (port), (addr), (count))
#define pvmnet_write(sc, port, addr, count) \
	bus_space_write_region_1((sc)->mem_bst, (sc)->mem_bsh, (port), (addr), (count))
